﻿using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using ServiceSecurity;
using ServiceSecurity.Data.Enumerations;

namespace OWL.ServiceSecurity.Attributes
{
    public class CheckApiTokenAttribute : ActionFilterAttribute
    {
        /// <summary>
        /// The tag which is used to check if the token has access to this specific endpoint
        /// </summary>
        public string EndpointTag;

        /// <summary>
        /// The name of the service in which the endpoint lies
        /// </summary>
        public eServiceNames ServiceName;

        private const string HttpKeyHeader = "x-api-key";
        #if DEBUG
        private const string PedamaUrl = "https://localhost:63796";
        #else
        private const string PedamaUrl = "https://pedama.dev.owl-live.de";
        #endif
        public CheckApiTokenAttribute(string EndpointTag, eServiceNames ServiceName)
        {
            this.EndpointTag = EndpointTag;
            this.ServiceName = ServiceName;
        }

        /// <summary>
        /// Before executing the endpoint-logic this method is called and validates if an API Key is provided and if it is valid for a specific endpoint
        /// </summary>
        /// <param name="context"></param>
        public override async Task OnActionExecutionAsync(ActionExecutingContext context, ActionExecutionDelegate next)
        {
            var headers = context.HttpContext.Request.Headers;

            if (headers.ContainsKey("Authorization"))
            {
                await next();
            }
            if (headers.ContainsKey(HttpKeyHeader))
            {
                var key = headers[HttpKeyHeader];

                // Checken ob der Key auch im StorageAccount abgelegt ist               
                var keyIsValid = await new KeyChecker().IsKeyValid(key, ServiceName.ToString(), EndpointTag);

                if (!keyIsValid)
                {
                    context.Result = new ContentResult()
                    {
                        StatusCode = 401,
                        Content = "API Key is not configured for this endpoint!"
                    };
                    return;
                }

                await next();
            }
            else
            {
                context.Result = new ContentResult()
                {
                    StatusCode = 401,
                    Content = "Required API Key was not provided!"
                };
                return;
            }
        }
    }
}
